@@ -196,6 +196,14 @@ def work3r(**kwargs):
 def discover(**kwargs):
     global devices
 
+    def join():
+        main_thread = threading.currentThread()
+        for t in threading.enumerate():
+            if t is main_thread:
+                continue
+            t.join()
+        return True
+
     print("Discovering devices...")
     try:
         if(kwargs['return_type'] == dict):
@@ -205,6 +213,11 @@ def discover(**kwargs):
     except:
         devices = {}
 
+    try:
+        additional_ranges = kwargs['additional_ranges']
+    except:
+        additional_ranges = []
+
     s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
     try:
         s.connect(('8.8.8.8', 1))
@@ -218,19 +231,26 @@ def discover(**kwargs):
         subnet = '.'.join(host_ip.split(".")[:3])+"."
         for i in range(1,255):
             threading.Thread(target=work3r,kwargs={"ip":subnet+str(i)}).start()
-            
-    main_thread = threading.currentThread()
-    for t in threading.enumerate():
-        if t is main_thread:
-            continue
-        t.join()
-    print("Dicovery complete!")
+
+    join()
+    
+    if(additional_ranges):
+        if(isinstance(additional_ranges,list)):
+            print("Looking in addtional ranges...")
+            for rng in additional_ranges:
+                print("{}...".format(rng))
+                for i in range(1,255):
+                    subnet = '.'.join(rng.split(".")[:3])+"."
+                    threading.Thread(target=work3r,kwargs={"ip":subnet+str(i)}).start()
+                join()
+
+    print("Dicovery complete!")    
     return devices
 
 def interactive():
     error_counter = 0
     response = False
-    found = discover()
+    found = discover(return_type=list)
 
     if(not found):
         try:
@@ -256,14 +276,9 @@ def interactive():
         except Exception as e:
             print('fail here')
             print(str(e))
-    else:
-        return False
     prompt = '''|-------------------------------------------------------------------------------------------------------------------|
 | Welcome to W3mo Pwn!                                                                                              |
 |-------------------------------------------------------------------------------------------------------------------|
-| I would like to personally thank Belkin for being a bag of di*** and hiding the API...                            |                  
-| Despite your best efforts, here we are!                                                                           |
-|-------------------------------------------------------------------------------------------------------------------|
 | To control your device just enter a state integer (0=OFF,1=ON)                                                    |
 | To get your device power state enter 'state'                                                                      |
 | To get your device's friendly name enter 'name'                                                                   |
@@ -278,9 +293,40 @@ def interactive():
 
 
     while True:
+        def calc_spaces(index,name):
+            TOTAL=112
+            out = ""
+            for i in range(TOTAL-(len(str(index))+len(name))):
+                out+=" "
+            return out
+
+        if(found):
+            discovery_prompt = '''|-------------------------------------------------------------------------------------------------------------------|
+| Device discovery found the following items, please select from the list                                           |
+|-------------------------------------------------------------------------------------------------------------------|'''
+            print(discovery_prompt)
+            for i in range(len(found)):
+                print("| {}: {}{}|".format(i,found[i]['name'],calc_spaces(i,found[i]['name'])))
+            print("|-------------------------------------------------------------------------------------------------------------------|")
+
+
         try:
-            
-            value = input('\nw3mo.sh3ll$ ')
+            term = input('\nw3mo.sh3ll$ ')
+            x = found[int(term)]['obj']
+        except:
+            try:
+                if(term.strip().lower() == 'exit'):
+                    break
+                else:
+                    print(prompt)
+            except Exception as e:
+                print("Failure! {} | {}".format(type(e).__name__,e.args))
+        try:
+            if(not x):
+                print("No device to control!")
+                break
+                
+            value = input('\nw3mo.sh3ll$({}) '.format(x.name))
             response = False
             try:
                 value = int(value)
